#!/usr/bin/python
# autoshade.py - AutoShade configure file
# for AutoShade v1.02

# methods:
#  log(string) => None
#  getWindowProperty(HWND) => dict
#  doShadeWindow(HWND, bool) => None
#  doTransparentWindow(HWND, bool) => None
#  doTopmostWindow(HWND, bool) => None
import AutoShade

def ShadableLogger(hwnd, title, classname):
	"""just log"""
	AutoShade.log(u"Hooked: <title:%s, class:%s, HWND:%u>" % (title, classname, hwnd))
	# dic = AutoShade.getWindowProperty(hwnd)
	# AutoShade.log(u"parent %(parent)d, owner %(owner)d, is_visible %(is_visible)d, is_enabled %(is_enabled)d" % dic)
	return False

class ShadableChecker(object):
	"""connect conditions by AND"""
	def __init__(self):
		self.conditions = []
	
	def byClass(self, match_classname):
		"""shade by class name matching"""
		def isShadableClassname(hwnd, title, classname):
			return match_classname==classname
		self.conditions.append(isShadableClassname)
		return self
	
	def byTitle(self, match_title):
		"""shade by window title matching"""
		def isShadableTitle(hwnd, title, classname):
			return match_title==title
		self.conditions.append(isShadableTitle)
		return self
	
	def byTitleR(self, match_title):
		"""shade by window title matching (regex)"""
		try:
			# Python 2.6 is required for "re" (or other modules)
			import re
			def isShadableTitle(hwnd, title, classname):
				return re.match(match_title, title)!=None
			self.conditions.append(isShadableTitle)
		except:
			pass
		return self
	
	def __call__(self, hwnd, title, classname):
		"""check if the window should be shaded"""
		result = True
		for cond in self.conditions:
			if not cond(hwnd, title, classname):
				result = False
				break
		if len(self.conditions)==0:
			result = False
		return result

def addWindowCallback(hwnd, title, classname):
	"""return True to add the window under control."""
	# AutoShade.log(u"addWindow: <title:%s, class:%s, HWND:%u>" % (title, classname, hwnd))
	return True

def removeWindowCallback(hwnd, title, classname):
	# AutoShade.log(u"removeWindow: <title:%s, class:%s, HWND:%u>" % (title, classname, hwnd))
	pass

def shadeWindowCallback(hwnd, title, classname):
	"""return True to shade, or return False not to shade."""
	# AutoShade.log(u"shadeWindow: <title:%s, class:%s, HWND:%u>" % (title, classname, hwnd))
	return True

def unshadeWindowCallback(hwnd, title, classname):
	"""return True to unshade, or return False not to unshade."""
	# AutoShade.log(u"unshadeWindow: <title:%s, class:%s, HWND:%u>" % (title, classname, hwnd))
	return True

def activateWindowCallback(hwnd, title, classname):
	"""this method is called when a window is activated."""
	# AutoShade.log(u"activateWindow: <title:%s, class:%s, HWND:%u>" % (title, classname, hwnd))
	AutoShade.doTopmostWindow(hwnd, False)
	AutoShade.doTransparentWindow(hwnd, False)
	AutoShade.doShadeWindow(hwnd, False)

def unactivateWindowCallback(hwnd, title, classname):
	"""this method is called when a window is unactivated."""
	# AutoShade.log(u"unactivateWindow: <title:%s, class:%s, HWND:%u>" % (title, classname, hwnd))
	AutoShade.doShadeWindow(hwnd, True)
	AutoShade.doTransparentWindow(hwnd, True)
	AutoShade.doTopmostWindow(hwnd, True)

def configure():
	"""return ([list of shadable checker callables],
		callback on addWindow,
		callback on removeWindow,
		callback on shadeWindow,
		callback on unshadeWindow,
		)"""
	ls = []
	ls.append(ShadableLogger) # logging
	ls.append(ShadableChecker().byClass(u"CabinetWClass")) # Explorer
	ls.append(ShadableChecker().byClass(u"MozillaUIWindowClass")) # Firefox
	ls.append(ShadableChecker().byClass(u"HH Parent")) # HTML Help
	#ls.append(ShadableChecker().byClass(u"PuTTY").byTitleR(u"^~")) # PuTTY in home directory
	
	AutoShade.log(u"configure() executed. adding %d filters." % len(ls))
	return (ls, addWindowCallback, removeWindowCallback,
		shadeWindowCallback, unshadeWindowCallback,
		activateWindowCallback, unactivateWindowCallback)
